using System;
using System.Linq;
using System.Reflection;

namespace Syntriax.Engine.Core.Factory;

public static class TypeFactory
{
    public static T Get<T>(params object?[]? args) where T : class => (T)Get(typeof(T), args);
    public static object Get(string fullName, params object?[]? args) => Get(GetType(fullName), args);
    public static object Get(Type type, params object?[]? args)
    {
        object? result;

        if (args is not null && args.Length != 0)
            result = Activator.CreateInstance(type, args);
        else
            result = Activator.CreateInstance(type);

        if (result is null)
            throw new Exception($"Type {type.Name} could not be created.");

        return result;
    }

    public static Type GetType(string fullName)
    {
        foreach (Assembly assembly in AppDomain.CurrentDomain.GetAssemblies())
            if (assembly.GetTypes().FirstOrDefault(t => t.FullName?.CompareTo(fullName) == 0) is Type type)
                return type;

        throw new Exception($"Type {fullName} could not be found in the current domain.");
    }
}
